package checkAlign;

public class Model {
	private Matrix matrix = new Matrix();
	private SearchTable stable = new SearchTable();
	private Schneider sch = new Schneider();
	private PostScript2 ps2 = new PostScript2();
	private IllegalChars ic = new IllegalChars();
	private boolean hasIllegalChars = false;
	private double[][] freqMatrix;
	private double[][] finalMatrix;
	private String psFile = "";
	private String[] psFiles;
	private int maxPosPerPage = 200;
	
	public Model(){
		
	}
	void setAlignment(String input, boolean protein){
		matrix.createMatrix(input, protein);
		freqMatrix = matrix.getFreqMatrix();
		ic.setMatrix(matrix.getMatrix());
		hasIllegalChars = ic.searchIllegalChars();
	}
	void calculate(int method, boolean protein, boolean gaps, boolean correction, boolean sd, int percent){
		double hg = 0;
		double eValue = 0;
		double sdValue = 0;
				
		if (method == 1){ // schneider			
			String tableLine = stable.getLine(matrix.getNumSeq(), freqMatrix.length, gaps);
    		eValue = stable.getE(tableLine);
    		sdValue = stable.getSd(tableLine);
    		
    		sdValue = Math.round((sdValue * 100) / getHg(freqMatrix.length, gaps));
    		
    		sch.calculateSchneider(freqMatrix, gaps, correction, eValue);
    		
    		finalMatrix = sch.getMatrix();
		}
		else if (method == 2){
			finalMatrix = freqMatrix;
		}
		
		if (!sd) sdValue = 0;
		
			
		// cuenta de posiciones a dibujar incluyendo el primer blanco
		boolean firstWhite = false;
		boolean draw = false;
		int drawCount = 0;
		double ratio = 0;
		int numGap = 0;
		
		
		if (method == 1) ratio = 0.01;
		else ratio = (percent/(double)100);
		
		if (protein) numGap = 20;
		else numGap = 4;
		
		for (int i=0; i<finalMatrix[0].length; i++){
			draw = false;
			for (int j=0; j<finalMatrix.length; j++){
				if (finalMatrix[j][i] >= ratio && j!=numGap){
					draw = true;
				}
			}
			if (draw){
				drawCount++;
				firstWhite = false;
			}else{
				if (!firstWhite){
					firstWhite = true;
					drawCount++;
				}
			}
		}
		
		// calculo de numero de documentos que se crearan
		int parts = drawCount / maxPosPerPage;
		int rest = drawCount % maxPosPerPage;
		int docNum = 0;
		
		if (parts == 0) docNum = 1;
		else{
			docNum = parts;
			if (rest > 0){
				docNum++;
			}
		}
		
		psFiles = new String[docNum];
		
		int indexIni = 0;
		int indexFin = 0;
		int docCounter = 0;
		
		drawCount = 0;
		// si hay cero documentos, se manda todo el array
		if (parts == 0){
			ps2.createFile(finalMatrix, method, getHg(freqMatrix.length, gaps), sdValue, percent, 0, finalMatrix[0].length);
			psFiles[docCounter++] = ps2.getFileText();
		}// si hay mas de cero, se manda en el for
		else{		
			for (int i=0; i<finalMatrix[0].length; i++){
				draw = false;
				for (int j=0; j<finalMatrix.length; j++){
					if (finalMatrix[j][i] >= ratio && j!=numGap){
						draw = true;
					}
				}
				if (draw){
					drawCount++;
					firstWhite = false;
				}else{
					if (!firstWhite){
						firstWhite = true;
						drawCount++;
					}
				}
				if (drawCount == maxPosPerPage){					
					drawCount = 0;
					indexFin = i;
					
					ps2.createFile(finalMatrix, method, getHg(freqMatrix.length, gaps), sdValue, percent, indexIni, indexFin+1);
					psFiles[docCounter++] = ps2.getFileText();
					indexIni = i+1;
				}
			}
			if (rest > 0){
				ps2.createFile(finalMatrix, method, getHg(freqMatrix.length, gaps), sdValue, percent, indexFin+1, finalMatrix[0].length);
				psFiles[docCounter++] = ps2.getFileText();
			}
		}		
	}
	String[] getPsFiles(){
		return psFiles;
	}
	double getHg(int numLetters, boolean gaps){
		double hg=0;
		
		if (!gaps){
			if (numLetters == 21 || numLetters == 5){
				numLetters--;
			}
		}
		
		switch (numLetters){
			case 4: hg=2; break;
			case 5: hg=2.3; break;
			case 20: hg=4.3; break;
			case 21: hg=4.4; break;
		}
		
		return hg;
	}
	boolean hasIllegalChars(){
		return hasIllegalChars;
	}
}