package checkAlign;

import javax.swing.*;
import javax.swing.border.Border;
import java.awt.*;
import java.awt.event.*;

public class View extends JFrame{
	// elements
	JMenuBar menuBar;
	JMenu menuFile, menuHelp;
	JMenuItem menuFileExit, menuHelpHelp, menuHelpAbout, menuHelpTest;
	Font fontNormal, fontBold, fontNote;
	JScrollPane scrollAlign;
	JTextArea textAlign;
	JRadioButton radioProtein, radioDNA;
	ButtonGroup bgType;
	JRadioButton radioMethod1, radioMethod2;
	ButtonGroup bgMethod;
	JCheckBox checkMethod1Gaps, checkMethod1Correction, checkMethod1Sd;
	JTextField checkMethod2Freq;
	JButton buttonSave, buttonLoad;
	JLabel labelStatus;
	Model model;
	
	public View(Model model){
		super("Check align");
		
		this.model = model;
		
		initComponents();
		
		Container cp = getContentPane();
		cp.setLayout(new BoxLayout(cp, BoxLayout.Y_AXIS));
		setJMenuBar(createMenuBar());
				
		cp.add(crateMainPanel());
		cp.add(labelStatus);	
	}
	
	// create interface
	private JMenuBar createMenuBar(){
		menuBar = new JMenuBar();
		menuFile = new JMenu("File");
		menuHelp = new JMenu("Help");
		menuFileExit = new JMenuItem("Exit");
		menuHelpHelp = new JMenuItem("Help");
		menuHelpAbout = new JMenuItem("About...");
		menuHelpTest = new JMenuItem("Test");
		
		menuBar.add(menuFile);
		menuBar.add(menuHelp);
		
		menuFile.add(menuFileExit);
		menuHelp.add(menuHelpHelp);
		menuHelp.add(menuHelpAbout);
		//menuHelp.add(menuHelpTest);
		
		menuFile.setFont(fontNormal);
		menuHelp.setFont(fontNormal);
		menuFileExit.setFont(fontNormal);
		menuHelpHelp.setFont(fontNormal);
		menuHelpAbout.setFont(fontNormal);
		
		return menuBar;
	}
	private void initComponents(){		
		// fonts
		fontNormal = new Font("Sans-seriff", Font.PLAIN, 12);
		fontBold = new Font("Sans-seriff", Font.BOLD, 12);
		fontNote = new Font("Sans-seriff", Font.PLAIN, 10);
		
		// panel sup
		scrollAlign = new JScrollPane(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		textAlign = new JTextArea(20,50);
		
		// type
		radioProtein = new JRadioButton("Protein");
		radioDNA = new JRadioButton("DNA");
		bgType = new ButtonGroup();
		
		// methods
		radioMethod1 = new JRadioButton("<html>Generate a logos from a DNA or protein alignment, using the information theory-based method<br>" +
		" of Tom Schneider with the Shannon algorithm.</html>");
		radioMethod2 = new JRadioButton("<html>Generate a frequency-based logos from a gapped and ungapped DNA or protein alignment," +
		"<br> only using the relative frequencies.</html>");
		bgMethod = new ButtonGroup();
		
		checkMethod1Gaps = new JCheckBox("Include gaps", true);
		checkMethod1Correction = new JCheckBox("Correction factor", false);
		checkMethod1Sd = new JCheckBox("Std. dev. bars");
		checkMethod2Freq = new JTextField(3);
		
		buttonSave = new JButton("Create Logos");
		buttonLoad = new JButton("Search file");
		
		labelStatus = new JLabel("Load a file");
		labelStatus.setFont(fontNormal);	
	}	
	private JPanel crateMainPanel(){
		JPanel panelMain = new JPanel();
		panelMain.setLayout(new BoxLayout(panelMain, BoxLayout.Y_AXIS));
		
		panelMain.add(createPanelSup());
		panelMain.add(createPanelCenter());
		panelMain.add(createPanelExport());
		
		return panelMain;
	}
	private JPanel createPanelSup(){
		JPanel panelSup = new JPanel();
		JPanel panelButton = new JPanel();
		
		panelSup.setLayout(new BoxLayout(panelSup, BoxLayout.Y_AXIS));	
		panelButton.setLayout(new FlowLayout());
		
		scrollAlign.getViewport().add(textAlign);
		textAlign.setLineWrap(true);
		textAlign.setWrapStyleWord(true);
		textAlign.setEditable(false);
		
		bgType.add(radioProtein);
		bgType.add(radioDNA);
		radioProtein.setFont(fontNormal);
		radioDNA.setFont(fontNormal);
		radioProtein.setSelected(true);
		
		panelButton.add(buttonLoad);
		panelButton.add(radioProtein);
		panelButton.add(radioDNA);
		buttonLoad.setFont(fontNormal);
		
		panelSup.add(scrollAlign);
		panelSup.add(panelButton);
		
		Border border = BorderFactory.createLineBorder(Color.DARK_GRAY);
		Border borderTitle = BorderFactory.createTitledBorder(border, "1. Load an alignment: ");
		panelSup.setBorder(borderTitle);
		
		return panelSup;
	}
	private JPanel createPanelCenter(){
		JPanel panel = new JPanel();
		
		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
		
		bgMethod.add(radioMethod1);		
		bgMethod.add(radioMethod2);
		radioMethod1.setSelected(true);
		
		panel.add(createPanelMethod1());
		panel.add(createPanelMethod2());
		
		Border border = BorderFactory.createLineBorder(Color.DARK_GRAY);
		Border borderTitle = BorderFactory.createTitledBorder(border, "2. Select a method: ");
		panel.setBorder(borderTitle);
		
		return panel;
	}
	private JPanel createPanelMethod1(){
		JPanel panel = new JPanel();
		JPanel panelRadio = new JPanel();
		JPanel panelOptions = new JPanel();
		JPanel panelNote = new JPanel();
		JLabel labelMethod1Note = new JLabel("(Correction factor and error bars applicable up to 200 sequences)");

		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
		
		panelRadio.setLayout(new FlowLayout(FlowLayout.LEFT));
		panelOptions.setLayout(new FlowLayout(FlowLayout.LEFT));
		panelNote.setLayout(new FlowLayout(FlowLayout.LEFT));
		
		panelRadio.add(radioMethod1);
		
		panelOptions.add(new JLabel("                    "));
		panelOptions.add(checkMethod1Gaps);		
		panelOptions.add(checkMethod1Correction);
		panelOptions.add(checkMethod1Sd);
		
		checkMethod1Gaps.setFont(fontNormal);
		checkMethod1Correction.setFont(fontNormal);
		checkMethod1Sd.setFont(fontNormal);
		
		labelMethod1Note.setFont(fontNote);
		panelNote.add(new JLabel("                    "));
		panelNote.add(labelMethod1Note);
		
		panel.add(panelRadio);		
		panel.add(panelOptions);		
		panel.add(panelNote);
		
		radioMethod1.setFont(fontNormal);
		
		return panel;
	}
	private JPanel createPanelMethod2(){
		JPanel panel = new JPanel();
		JPanel panelRadio = new JPanel();
		JPanel panelNote = new JPanel();
		JPanel panelOptions = new JPanel();
		
		JLabel labelMethod2Note = new JLabel("<html>(Only recommended for preliminary analyses of non-conserved <br>protein alignments containing more than 100 sequences).</html>");
		JLabel labelMethod2Freq = new JLabel("Minimum frequency to be listed:");
		JLabel labelMethod2Def = new JLabel("% (default 10 percent)");
		
		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
		panelRadio.setLayout(new FlowLayout(FlowLayout.LEFT));
		panelNote.setLayout(new FlowLayout(FlowLayout.LEFT));
		panelOptions.setLayout(new FlowLayout(FlowLayout.LEFT));		
		
		radioMethod2.setFont(fontNormal);
		
		panelRadio.add(radioMethod2);
		
		panelNote.add(new JLabel("                    "));
		panelNote.add(labelMethod2Note);
		
		panelOptions.add(new JLabel("                    "));
		panelOptions.add(labelMethod2Freq);
		panelOptions.add(checkMethod2Freq);
		panelOptions.add(labelMethod2Def);
		
		checkMethod2Freq.setText("10");
		
		labelMethod2Note.setFont(fontNote);
		labelMethod2Freq.setFont(fontNormal);
		labelMethod2Def.setFont(fontNormal);
		
		panel.add(panelRadio);
		panel.add(panelNote);
		panel.add(panelOptions);
		
		return panel;
	}
	private JPanel createPanelExport(){
		JPanel panel = new JPanel();
		panel.setLayout(new FlowLayout());
		
		Border border = BorderFactory.createLineBorder(Color.DARK_GRAY);
		Border borderTitle = BorderFactory.createTitledBorder(border, "3. Generate logos: ");
				
		panel.setBorder(borderTitle);
		panel.add(buttonSave);
		buttonSave.setFont(fontNormal);
		
		return panel;
	}
	
	// listeners
	public void addLoadListener(ActionListener al){
		buttonLoad.addActionListener(al);
	}
	public void addSaveListener(ActionListener al){
		buttonSave.addActionListener(al);
	}
	public void addAboutListener(ActionListener al){
		menuHelpAbout.addActionListener(al);
	}
	public void addExitListener(ActionListener al){
		menuFileExit.addActionListener(al);
	}
	public void addTestListener(ActionListener al){
		menuHelpTest.addActionListener(al);
	}
}
